// sharedPtr2.cpp
// shared_ptr in einem STL-Contaier
// -----------------------------------------------------
#include <iostream>
#include <list>
#include <algorithm>
#include <memory>
using namespace std;

struct Shape                     // Eine Figur
{  int id;     // ID der Figur
   Shape(int _id) : id(_id) {}
   ~Shape() { cout << "Shape-Destruktor fuer id = " << id << endl; }
   virtual void draw() const = 0;
};

struct Rectangle : public Shape      // Rechteck
{
   Rectangle(int id): Shape(id) {}
   void draw() const { cout << "Rechteck " << id << endl; } 
};

struct Circle : public Shape         // Kreis
{
   Circle(int id) : Shape(id) {}
   void draw() const { cout << "Kreis    " << id << endl; } 
};

typedef shared_ptr<Shape> ShapePtr;

void shapePtrDraw( const ShapePtr& p) { p->draw(); }

int main()
{
  list<ShapePtr>  sPtrList;      // Liste mit Zeigern auf Figuren.
  list<ShapePtr>::iterator pos;  // Ein Iterator

  sPtrList.push_back( ShapePtr(new Rectangle(3)) );
  sPtrList.push_back( ShapePtr(new Circle(5)) );
  sPtrList.push_front( ShapePtr(new Rectangle(0)) );

  // Zeiger auf Rechteck nach erstem Element einfgen: 
  pos = sPtrList.begin();  ++pos;
  sPtrList.insert(pos, ShapePtr(new Circle(2)) );

  if( !sPtrList.empty() )
  {	  cout << "Erstes Element: "; sPtrList.front()->draw();
      cout << "wird geloescht." << endl;
      sPtrList.pop_front();      // Lscht Rectangle(0).
  }
  cout << "\nFiguren in der Liste: " << endl;
  for_each( sPtrList.begin(), sPtrList.end(), shapePtrDraw);
                        // Circle(2), Rectangle(3), Circle(5).
  sPtrList.clear();     // Alle Zeiger in der Liste lschen
                        // => Alle Figuren werden gelscht.
  return 0;
}

/* Ausgabe:
Erstes Element: Rechteck 0
wird geloescht.
Shape-Destruktor fuer id =  0

Figuren in der Liste:
Kreis    2
Rechteck 3
Kreis    5
Shape-Destruktor fuer id = 2
Shape-Destruktor fuer id = 3
Shape-Destruktor fuer id = 5
*/
