// shared_ptr_STL.cpp
// shared_ptr in einem STL-Contaier
// -----------------------------------------------------
#include <iostream>
#include <vector>
#include <set>
#include <algorithm>
#include <memory>
using namespace std;

struct Data
{
  float x;                   // Messwert
  // Weitere Daten wie Uhrzeit, Datum, ...
  Data( float xx = 0.0F ) : x(xx) {}
  ~Data() { cout << "Destruktor fuer x = " << x << endl; }
  // Weitere Methoden
};

inline bool operator<(const Data& d1, const Data& d2)
{  return d1.x < d2.x;  }

typedef shared_ptr<Data> DataPtr;

// Operationen fr DataPtr
inline bool operator<( const DataPtr& p1, const DataPtr& p2)
{  return *p1 < *p2;  }

void printData( const DataPtr& p) 
{ cout.width(8); cout << p->x; }

void printCount( const DataPtr& p) 
{ cout.width(8); cout << p->x << " : " << p.use_count(); }

int main()
{
   vector<DataPtr>  dataPtr_vec;

   // Einige Daten am Ende einfgen:
   dataPtr_vec.push_back( DataPtr(new Data(2.2F)) );
   dataPtr_vec.push_back( DataPtr(new Data(3.3F)) );
   dataPtr_vec.push_back( DataPtr(new Data(1.1F)) );
   dataPtr_vec.push_back( DataPtr(new Data(3.3F)) );

   cout << "Die Daten im vector-Container:" << endl;
   for_each( dataPtr_vec.begin(), dataPtr_vec.end(), printData);
   cout << endl;

   // Ein multiset-Container mit den Daten: (Daten knnen gleich sein.)
   multiset<DataPtr>  dataPtr_mset( dataPtr_vec.begin(), dataPtr_vec.end());

   cout << "Die Daten im multiset-Container:" << endl;
   for_each( dataPtr_mset.begin(), dataPtr_mset.end(), printData);
   cout << endl;

   dataPtr_vec.pop_back();       // Im Vektor die zwei letzten
   dataPtr_vec.pop_back();       // Elemente lschen.

   cout << "Die Anzahl der Besitzer nach dem Loeschen:" << endl;
   for_each( dataPtr_mset.begin(), dataPtr_mset.end(), printCount);
   cout << endl;

   dataPtr_mset.clear();   // Liste lschen => Destruktoren der Elemente, 
                           // die auch im Vektor gelscht wurden.
   cout << "Ende von main()." << endl;
   return 0;
}

/* Ausgabe:

Die Daten im vector-Container:
     2.2     3.3     1.1     3.3
Die Daten im multiset-Container:
     1.1     2.2     3.3     3.3
Die Anzahl der Besitzer nach dem Loeschen:
     1.1 : 1     2.2 : 2     3.3 : 2     3.3 : 1
Destructor fuer x = 3.3
Destructor fuer x = 1.1
Ende von main().
Destructor fuer x = 2.2
Destructor fuer x = 3.3
*/
