// Euro.h
// Die Klasse Euro zur Darstellung eines Euro
// mit arithmetischen Operatoren.
// Globale Operatorfunktionen fr +, - und *.
// Fr Betrge bis zu +- 20 Millionen Euro.
// ---------------------------------------------------

#ifndef _EURO_H_
#define _EURO_H_

#include <iostream>
#include <sstream>             // Fr Klasse stringstream
#include <iomanip>
using namespace std;
 
class Euro
{
  private:
    long  data;            // Euros * 100 + Cents

   public:
    Euro( int euro = 0, int cents = 0)
    {
       if( euro < 0)               // Euro(-1,20) == Euro(-1.20)
          cents = -cents; 
       data = 100L * (long)euro + cents;
    }

    Euro( double x)
    {
       x *= 100.0;                               // Runden, z.B.
       data = (long)( x >= 0.0 ? x+0.5 : x-0.5); // 9.7 -> 10
    }

    long getWholePart() const { return data/100; }
    int  getCents() const { return data%100; }

    double asDouble() const { return (double)data/100.0; }

    string asString() const     // Euro-Wert als String zurckgeben.
    {
       stringstream strStream;         // Stream zur Konvertierung
       long temp = data;
       if( temp < 0) { strStream << '-';  temp = -temp; }
       strStream << temp/100 << ','
                 << setfill('0') << setw(2) << temp%100;
       return strStream.str();
    }

    // ---- Operatorfunktionen ----

    Euro operator-() const              // Negation (unres Minus))
    {
       Euro temp;
       temp.data = -data;
       return  temp;
    }
    
    Euro& operator+=( const Euro& e2)   // Euros hinzuaddieren.
    {
       data += e2.data;
       return  *this;
    } 

    Euro& operator-=( const Euro& e2)   // Euros subtrahieren.
    {
       data -= e2.data;
       return  *this;
    } 

    Euro operator/( double x)           // Division *this / x
    {                                   // =  *this * (1/x)
       return (*this * (1.0/x));
    }

    friend Euro operator*( const Euro& e, double x)
    {
       Euro temp( ((double)e.data/100.0) * x) ;
       return temp;
    }

    friend Euro operator*( double x, const Euro& e) 
    {
       return e * x;
    }
};

// Addition:
inline Euro operator+( const Euro& e1, const Euro& e2)
{
    return Euro( e1.asDouble() + e2.asDouble());  
}
inline Euro operator-( const Euro& e1, const Euro& e2)
{
    return Euro( e1.asDouble() - e2.asDouble());  
}

// Deklaration der Ein-/Ausgabe-Operatoren:

// Auf Stream os ausgeben.
ostream& operator<<(ostream& os, const Euro& e);

// Von Stream is einlesen.
istream& operator>>(istream& is, Euro& e);

#endif   // _EURO_H_
