// Euro1.h
// Die Klasse Euro zur Darstellung eines Euro
// mit arithmetischen Operatoren.
// Fr Betrge bis zu +- 20 Millionen Euro.
// ---------------------------------------------------
#ifndef _EURO_H_
#define _EURO_H_

#include <sstream>             // Fr Klasse stringstream
#include <iomanip>
using namespace std;

class Euro
{
  private:              
    long  data;            // Euros * 100 + Cents

   public:
    Euro( int euro = 0, int cents = 0)
    {
       data = 100L * (long)euro + cents;
    }

    Euro( double x)
    {
       x *= 100.0;                               // Runden, z.B.
       data = (long)( x >= 0.0 ? x+0.5 : x-0.5); // 9.7 -> 10
    }

    long getWholePart() const { return data/100; }
    int  getCents() const { return data%100; }

    double asDouble() const { return (double)data/100.0; }

    string asString() const     // Euro-Wert als String zurckgeben.
    {
       stringstream strStream;         // Stream zur Konvertierung
       long temp = data;
       if( temp < 0) { strStream << '-';  temp = -temp; }
       strStream << temp/100 << ','
                 << setfill('0') << setw(2) << temp%100;
       return strStream.str();
    }

    void print( ostream& os) const      // Auf Stream os ausgeben.
    {
       os << asString() << " Euro" << endl;
    }

    // ---- Operatorfunktionen ----

    Euro operator-() const              // Negation (unres Minus))
    {
       Euro temp;
       temp.data = -data;
       return  temp;
    } 
    
    Euro operator+( const Euro& e2) const  // Addition.
    {
       Euro temp;
       temp.data = data + e2.data;
       return  temp;
    } 

    Euro operator-( const Euro& e2) const  // Subtraktion.
    {
       Euro temp;
       temp.data = data - e2.data;
       return  temp;
    } 

    Euro& operator+=( const Euro& e2)   // Euros hinzuaddieren.
    {
       data += e2.data;
       return  *this;
    } 

    Euro& operator-=( const Euro& e2)   // Euros subtrahieren.
    {
       data -= e2.data;
       return  *this;
    } 
};

#endif   // _EURO_H_
