// ----------------------------------------------------------  
// index.h:  Enthlt die Definition der Klassen
//           IndexEntry zur Darstellung eines Indexeintrags, 
//           IndexFile  zur Darstellung einer Index-Datei und
//           IndexFileSystem zur Darstellung eines
//                           Index-Datei-Systems.
// ----------------------------------------------------------  

#ifndef _INDEX_H
#define _INDEX_H
#pragma warning( disable : 4290 )  // VC untersttzt keine Ausnahmespezifikation 

#include <fstream>
#include <iostream>
#include <string>
#include "konto.h"
using namespace std;

class IndexEntry
{
  private:
    long key;
    long recNr;

  public:
    IndexEntry(long k=0L, long n=0L) { key = k; recNr = n; }

    void setKey(long k) { key = k; }
    long getKey() const { return key; }
    void setPos(long n) { recNr = n; }
    long getPos() const { return recNr; }

    int recordSize() const
    { return sizeof(key) + sizeof(recNr); }

    fstream& write(fstream& ind ) const;
    fstream& read( fstream& ind);

    fstream& write_at(fstream& ind, long pos ) const;
    fstream& read_at( fstream& ind, long pos);

    void display()
    {
       cout << "Kto-Nr: " << key
            << "   Pos: " << recNr << endl;
    }
};

// ----------------------------------------------------------  
// Definition der Klasse IndexFile:

class IndexFile
{
  private:
    fstream index;
    string  name;              // Name der Indexdatei

  public:
    IndexFile(const string& s)  throw(open_error);
    ~IndexFile() { index.close(); }

    void insert( long key, long pos)  throw(read_error, write_error);
    long search( long key)            throw(read_error);
    void retrieve(IndexEntry& entry, long pos )  throw(read_error);

    void display() throw(read_error);
};

// ----------------------------------------------------------  
// Definition der Klasse IndexFileSystem:

class IndexFileSystem
{
   private:
     KontoFile konten;        // Primrdatei
     IndexFile index;         // Indexdatei

   public:
     IndexFileSystem(const string& s) 
     : konten(s + ".prim"), index(s + ".ind")
     { }

    bool   insert  ( const Konto& kto );
    Konto* retrieve( long key );

    void displayIndexFile() throw( read_error) { index.display(); }
    void displayKontoFile() throw( read_error) { konten.display(); }
};

#endif
