// ----------------------------------------------------
// konto.cpp
// Die Implementierung der Methoden der Klassen
// Konto, GiroKonto, Sparkonto und KontoFile.
// ----------------------------------------------------

#include "konto.h"

ostream& Konto::write(ostream& os) const
{
   os << name << '\0';
   os.write((char*)&nr, sizeof(nr) );
   os.write((char*)&stand, sizeof(stand) );
 
   return os;
}
 
istream& Konto::read(istream& is)
{
   getline( is, name, '\0');
   is.read((char*)&nr, sizeof(nr) );
   is.read((char*) &stand, sizeof(stand));
   
   return is;
}

ostream& GiroKonto::write(ostream& os) const
{
   if(!Konto::write(os))
       return os;
   os.write((char*)&limit, sizeof(limit) );
   os.write((char*)&soll, sizeof(soll) );
   
   return os;   
}
 
istream& GiroKonto::read(istream& is)
{
   if(!Konto::read(is))
       return is;
   is.read((char*)&limit, sizeof(limit) );
   is.read((char*)&soll, sizeof(soll));

   return is;
}

ostream& SparKonto::write(ostream& os) const
{
   if(!Konto::write(os))
       return os;
   os.write((char*)&haben, sizeof(haben) );
    
   return os;   
}
 
istream& SparKonto::read(istream& is)
{
   if(!Konto::read(is))
       return is;
   is.read((char*)&haben, sizeof(haben) );

   return is;
}

// ---------------------------------------------------
// Methoden der Klasse KontoFile
// ---------------------------------------------------

KontoFile::KontoFile(const string& s) throw( open_error)
{
   ios::openmode mode = ios::in | ios::out | ios::app | ios::binary;
   f.open( s.c_str(), mode );
   
   if(!f)
      throw open_error(s);
   else  
      name = s;
}

void KontoFile::display() throw( read_error)
{
   Konto  kto, * kto_p = NULL;
   GiroKonto giroKto;
   SparKonto sparKto;
   TypeId id;

   if( !f.seekg(0L))
      throw read_error(name);
     
   cout << endl << "Die Konten-Datei: " << endl;

   while( f.read((char*)&id, sizeof(TypeId)) )
   {
      switch(id)
      {
        case KONTO:  kto_p = &kto;
                     break; 
        case GIRO:   kto_p = &giroKto;
                     break; 
        case SPAR:   kto_p = &sparKto;
                     break; 

        default: cerr << "Falsches Flag in Kontendatei" << endl; 
                 exit(1);
      } 

      if(!kto_p->read(f))
         break;

      kto_p->display();
      cin.get();          // Weiter mit Return
   }
 
   if( !f.eof())
       throw read_error(name);
    
   f.clear();
}

long KontoFile::append( const Konto& kto) throw( write_error)
{
   f.seekp(0L, ios::end);         // Schreibzeiger positionieren
   long pos = (long)f.tellp();    // und sichern.
   
   if( !f )
      throw write_error(name);
   
   TypeId id = kto.getTypeId();
   f.write( (char*)&id, sizeof(id));  // TypeId in die Datei
                                      // schreiben.
   if(!f)
       throw write_error(name);
   else
       kto.write(f);           // Objekt in Datei schreiben. 
     
   if(!f)
       throw write_error(name);

   return pos;
}
   
Konto* KontoFile::retrieve( long pos) throw(read_error)
{
   f.clear();
   f.seekg(pos);         // Lesezeiger positionieren
   
   if( !f )
      throw read_error(name);

   TypeId id;
   f.read( (char*)&id, sizeof(id) );
  
   if(!f)
      throw read_error(name);

   Konto* buf;
   switch( id )
   {
      case KONTO:  buf = new Konto;
                   break;
      case SPAR:   buf = new SparKonto; 
                   break;
      case GIRO:   buf = new GiroKonto; 
                   break;
      default:     return NULL;
   }
   if(!(buf->read(f)))
       throw read_error(name);

   return buf;
}
