// -------------------------------------------------------
// vektor.h
// Darstellung von Vektoren durch Klassen-Templates.
// ------------------------------------------------------
#ifndef _VEKTOR_H
#define _VEKTOR_H

#include <iostream>
#include <iomanip>
using namespace std;

#pragma warning( disable : 4290 )  // VC untersttzt keine Ausnahmespezifikation 

class BadIndex
{
  private:
    int index;
  public:
    BadIndex(int i):index(i){}
    int getBadIndex() const { return index; }
};

class OutOfRange
{
  // Ohne Datenelemente
};

template <class T, int n = 256>
class Vektor
{
   private:
     T   vek[n];          // Der Vektor
     int anz;             // Aktuelle Anzahl Elemente
 
   public:
     Vektor( ){ anz = 0;}

     Vektor(int n, const T& wert );

     int  length() const { return anz; }
     int  size()   const { return n; }

     T& operator[](int i) throw(BadIndex)
     {
        if( i < 0 || i >= anz ) throw BadIndex(i);

        return vek[i];
     }

     const T&  operator[](int i) const throw(BadIndex)
     {
        if( i < 0 || i >= anz ) throw BadIndex(i);

        return vek[i];
     }

     Vektor& operator+=( float wert) throw(OutOfRange)
     {
        append( wert);   return *this;
     }

     Vektor& operator+=(const Vektor& v)  throw(OutOfRange)
     {
        append(v);   return *this;
     }

     void append( T wert) throw(OutOfRange);
     void append( const Vektor& v) throw(OutOfRange);

     void insert( T wert, int pos) throw(BadIndex, OutOfRange);
     void insert( const Vektor& v, int pos ) throw(BadIndex, OutOfRange);

     void remove(int pos) throw(BadIndex);
};


template <class T, int n >
Vektor<T,n>::Vektor(int m, const T& wert ) 
{ 
   anz = m;
   for(int i=0; i < anz; i++ )
      vek[i] = wert;
}

template <class T, int n >
void Vektor<T,n>::append( T wert) throw(OutOfRange)
{ 
   if( anz < n)
       vek[anz++] = wert;
   else
       throw OutOfRange();
}

template <class T, int n >
void Vektor<T,n>::append( const Vektor<T,n>& v) throw(OutOfRange)
{ 
   if( anz + v.anz > n)           // Nicht genug Platz
      throw OutOfRange();
    
   int anzahl = v.anz;            // falls v == *this
                                  // notwendig.
   for( int i=0; i < anzahl; ++i)
     vek[anz++] = v.vek[i];
}


template <class T, int n >
void Vektor<T,n>::insert( T wert, int pos) throw(BadIndex, OutOfRange)
{
   insert( Vektor<T,n>(1,wert), pos);
}

template <class T, int n >
void Vektor<T,n>::insert( const Vektor<T,n>& v, int pos ) 
                                           throw(BadIndex, OutOfRange)
{
   if( pos < 0 || pos >= anz)
      throw BadIndex();              // unzulssige Position.

   if( n < anz + v.anz)
      throw OutOfRange();
 
   int i;
   for( i = anz-1; i >= pos; --i)    // Ab pos nach oben
      vek[i+v.anz] = vek[i];         // schieben.

   for( i = 0; i < v.anz; ++i)       // Lcke fllen.
      vek[i+pos] = v.vek[i];
   anz = anz + v.anz;
}


template <class T, int n >
void Vektor<T,n>::remove(int pos) throw(BadIndex)
{ 
   if( pos >= 0 && pos < anz)
   {
      for( int i = pos; i < anz-1; ++i) 
         vek[i] = vek[i+1];
      --anz;
   }
   else
     throw BadIndex(pos);
}

template <class T, int n >
ostream& operator<<( ostream& os, const Vektor<T,n>& v)
{
   int w = (int)os.width();         // Feldbreite merken.
   
   for( int i = 0; i < v.length(); ++i)
   {
      os.width(w);    cout << v[i];
   }
   os << endl;
   return os;
}

#endif
