// ------------------------------------------------------
// FloatVek.h : Darstellung dynamischer float-Vektoren.
// Methoden werfen eine Exception bei unzulssigem Index.
// ------------------------------------------------------
#ifndef _FLOATVEK_
#define _FLOATVEK_
#pragma warning( disable : 4290 )  // VC untersttzt keine Ausnahmespezifikation 

#include <iostream>
using namespace std;

class BadIndex
{
  private:
   int index;
  public:
    BadIndex(int i){index = i;}
    int getBadIndex() const {return index;}
};

class FloatVek
{
   private:
     float* vekPtr;    // Dynamisches Element
     int max;          // Maximale Anzahl, ohne
                       // erneut Speicher zu resevieren.
     int anz;          // Aktuelle Anzahl Elemente

     void expand( int neueGroesse);  // Hilfsfunktion, die
                                     // den Vektor vergrert.
   public:
     FloatVek( int n = 256 );
     FloatVek( int n, float wert);
     
     FloatVek(const FloatVek& src);           // Kopierkonstruktor
     FloatVek(FloatVek&& src);                // Move-Konstruktor
     
     ~FloatVek();

     FloatVek& operator=( const FloatVek&);   // Zuweisung
     FloatVek& operator=(FloatVek&& src);     // Move-Zuweisung
     
     int  length() const { return anz; }
 
      // Index-Operatoren:
     float& operator[](int i) throw(BadIndex); 
     float operator[](int i) const  throw(BadIndex); 

     // Methoden zum Anhngen eines float-Wertes 
     // oder eines float-Vektors:
     void append( float wert);
     void append( const FloatVek& v); 

     // Methoden zum Einfgen eines float-Wertes 
     // oder eines float-Vektors:
     void insert( float wert, int pos) throw(BadIndex); 
     void insert( const FloatVek& v, int pos ) throw(BadIndex); 
 
     void remove(int pos) throw(BadIndex);  // Position pos lschen.

    // Einen zweiten Vektor elementweise hinzuaddieren:
    FloatVek& operator+=( const FloatVek& v2);

    void swap( FloatVek& v2)       // *this mit v2 Tauschen.
    {
       if( this != &v2 )
       {  // Datenelemente tauschen:
          float* p = vekPtr;  vekPtr = v2.vekPtr;  v2.vekPtr = p;
          int i;
          i = max;   max = v2.max;  v2.max = i;
          i = anz;   anz = v2.anz;  v2.anz = i;
       }
    }

     // Ausgabe des Vektors
    friend ostream& operator<<( ostream& os, const FloatVek& v);
};

// Globale Funktion zum Tauschen:
inline void swap( FloatVek& v1, FloatVek& v2)     // v1 mit v2 Tauschen.
{   v1.swap(v2);   }

// Elementweise Addition zweier Vektoren:
FloatVek operator+( const FloatVek& v1, const FloatVek& v2);

#endif   // _FLOATVEK_