// ------------------------------------------------------ 
// konto.h 
// Definition der Klassen 
//       Konto, GiroKonto und SparKonto
// mit virtuellen read- und write-Methoden sowie
// der Klasse KontoFile zur Darstellung von Konto-Dateien.
// ------------------------------------------------------

#ifndef _KONTO_H
#define _KONTO_H
#pragma warning( disable : 4290 )  // VC untersttzt keine Ausnahmespezifikation 

#include <fstream>
#include <iostream>
#include <iomanip>
#include <string> 
using namespace std;

#include "exceptio.h"

enum TypeId { KONTO, GIRO, SPAR };

class Konto
{
   private:
     string name;
     unsigned long nr;
     double stand;

   public:                          // Konstruktor:
     Konto( const string& k_name = "X",
            unsigned long k_nr   = 1111111L,
            double  k_stand      = 0.0)
     :name(k_name), nr(k_nr), stand(k_stand)    
     { }

     virtual ~Konto() {}            // Virtueller Destruktor

     // Die get Methoden:
     const string& getName()  const { return name; }
     unsigned long getNr()    const { return nr; }
     double        getStand() const { return stand; }

     // Die set Methoden:
     bool setName( const string& s)
     {
       if( s.size() < 1)     // Kein leerer Name
         return false;
       name = s;
         return true; 
     }
     void setNr( unsigned long n) { nr = n; }
     void setStand(double x) { stand = x; }

     // Virtuelle Methoden:
     virtual TypeId getTypeId() const { return KONTO; }

     virtual istream& read( istream& is);
     virtual ostream& write( ostream& os) const;
      
     virtual void display() const
     { 
        cout << fixed << setprecision(2)
             << "----------------------------------\n"
             << "Kontoinhaber:       " << name  << endl
             << "Kontonummer:        " << nr    << endl
             << "Kontostand:         " << stand << endl
             << "----------------------------------"
             << endl;
     }
};
 
class GiroKonto : public Konto
{
   private:
     double limit;           // berziehungslimit
     double soll;            // Soll-Zinssatz

   public:
     GiroKonto( const string s = "X", unsigned long n = 1111111L,
                double st = 0.0, double li = 0.0, double so = 0.0)
     : Konto(s, n, st), limit(li), soll(so)
     { } 

     // Zugriffsmethoden:
     double getLimit() const { return limit; }
     void   setLimit(double lt){ limit = lt; }
    
     double getSoll() const { return soll; }
     void   setSoll(double sl){ soll = sl; }

	 TypeId getTypeId() const override { return GIRO; }

	 ostream& write(ostream& fs) const override;
	 istream& read(istream& fs) override;
      
	 void display() const override
     {   
         Konto::display();
         cout << "Ueberziehungslimit: " << limit << endl
              << "Soll-Zinssatz:      " << soll  << endl
              << "----------------------------------\n"
              << endl;
     }
};
 
class SparKonto: public Konto
{
   private:
     double haben;            // Habens-Zinssatz

   public:
     SparKonto(const string& s = "X", unsigned long n = 1111111L,
               double st = 0.0, double ha = 0.0)
     : Konto(s, n, st), haben(ha)
     { } 

    // Zugriffsmethoden.
    double getHaben() const { return haben; }
    void   setHaben(double hb){ haben = hb; }

	TypeId getTypeId() const override { return SPAR; }

	ostream& write(ostream& fs) const override;
	istream& read(istream& fs) override;
      
	void display() const override
    {   
        Konto::display();
        cout << "Haben-Zinssatz:     " << haben << endl
             << "----------------------------------\n"
             << endl;
    }
};

// -----------------------------------------------------------
// Definition der Klasse KontoFile

class KontoFile
{
   private:
     fstream f;             // File stream
     string  name;          // Dateiname

   public:
       KontoFile(const string& s) throw(open_error);
       ~KontoFile() { f.close(); }

       long   append( const Konto& kto)  throw(write_error);
       Konto* retrieve( long pos ) throw(read_error);

       void display() throw( read_error);
};

#endif
