// matrix.h:         Darstellung von Matrizen
// -----------------------------------------------------
#include <valarray>
#include <stdexcept>
using namespace std;

template<class T>
class Matrix
{
  private:
    valarray<T> element;   // Numerischer Vektor
    int rows, cols;        // Anzahl Zeilen und Spalten.
  public:
    Matrix(int n, int m, const T& a = T(0))
         : rows(n), cols(m), element(a, n*m){}
    Matrix(const T *v, int n, int m)
         : rows(n), cols(m), element(v, n*m){}
    Matrix(const valarray<T>& v, int n, int m)
         : rows(n), cols(m), element(v){}
/*
    Matrix(const Matrix& mat) : rows(mat.rows),
           cols(mat.cols), element(mat.element){}
*/
    int getRows() const { return rows; } 
    int getCols() const { return cols; } 

    T& operator()(int i, int j)        // Element in i-ter Zeilen
    {                                  // j-ter Spalte.
        if( i >= 0 && i < rows && j >= 0 && j < cols)
          return element[i*cols + j];   
        else
          throw(out_of_range("Unzulssiger Index!"));
    }

    valarray<T> operator[](int i) const         // i. Zeilenvektor
    { 
      if( i >= 0 && i < rows)
          return element[slice(i*cols, cols, 1)];
      else
         throw(out_of_range("Unzulssiger Index!"));
    }

    // Zuweisung, Operatoren etc.

    Matrix subMatrix(int is, int js, int ie, int je)
    {
      if( is <= ie && ie < rows && js <= je && je < cols)
      {
        int srows =  ie - is + 1, scols = je - js + 1;
        size_t vlen[] = { srows, scols};
        size_t vdist[] = {cols, 1};
        valarray<size_t> len(vlen, 2), dist(vdist, 2);  
        valarray<T> gsl(element[gslice(is*cols+js, len, dist)]);
        return Matrix(gsl, srows, scols);
      }
      else throw(out_of_range("Unzulssiger Index!"));
    }
  
  Matrix minor(int i,int j)
  {
    if(i >= 0 && i < rows && j >= 0 && j < cols)
    {
      size_t vdist[] = {cols-1, 1};      // Schrittweiten fr neue
      valarray<size_t> dist(vdist, 2), len(2);   // Matrix.
      Matrix<T> adj(rows - 1, cols - 1); // Teilmatrix:
      if(i != 0 && j !=  0)              // links oberhalb [i,j]
      {
        len[0] = i; len[1] = j;  
        Matrix<T> subMat = subMatrix(0,0,i-1,j-1);
        adj.element[gslice(0, len, dist)] = subMat.element;
      }
      if( i != 0 && j != cols-1)         // rechts oberhalb [i,j] 
      { 
        len[0]= i; len[1] = cols - j - 1;
        Matrix<T> subMat = subMatrix(0,j+1,i-1,cols-1);
        adj.element[gslice(j, len, dist)] = subMat.element;
      }
      if(i != rows-1 && j != 0)          // links unterhalb [i,j]
      {
        len[0]= rows - i - 1; len[1] = j;
        Matrix<T> subMat = subMatrix(i+1,0,rows-1,j-1);
        adj.element[gslice(i*(cols-1), len, dist)] = subMat.element;
      }
      if( i!=rows-1  && j != cols-1)     // rechts unterhalb [i,j]
      {
        len[0]= rows - i - 1; len[1] = cols - j - 1;
        Matrix<T> subMat= subMatrix(i+1,j+1,rows-1,cols-1);
        adj.element[gslice(i*(cols-1)+j,len,dist)] = subMat.element;
      }
      return adj;
    }
    else throw out_of_range("Unzulssiger Index!");
  }

   Matrix transponierte()
   {
      size_t* v = new size_t[cols*rows];
      for(int i = 0; i < rows; i++)
         for(int j = 0; j < cols; j++)
             v[j*rows + i] = i*cols + j;
      valarray<size_t> indirect(v, rows*cols);
      delete v;
                         // Indirekte Indizierung:
      Matrix trans(element[indirect], cols, rows);
      return trans;
   }

   Matrix& upperTriangle(const Matrix& m)
   {                           // Matrizen kompatibel?
     if( rows == cols)
     {                    
      valarray<bool> mask(false, cols*cols);  // Maske:
      for(int i = 0; i < rows; i++)   // Obere Dreiecksmatrix
         for(int j = i+1; j < rows; j++)
           mask[i*cols + j] = true;  
                                     // Maskierte Indizierung:
      element[mask] = m.element; 
      return *this;
     }
     else  
      throw(length_error("Matrix nicht quadratisch"));
   }
};
