// konto.h 
// Definition der Klasse Konto mit den 
// Methoden read() und write().
// ---------------------------------------------------

#ifndef _KONTO_
#define _KONTO_

#include <iostream>
#include <iomanip>
#include <string>
using namespace std;

class Konto
{
  private:            // Geschtzte Elemente:
     string name;             // Kontoinhaber
     unsigned long nr;        // Kontonummer
     double stand;            // Kontostand
  public:             // ffentliche Schnittstelle:
                      // Konstruktoren, Destruktor:
     Konto( const string& k_name = "X",
            unsigned long k_nr   = 1111111L,
            double  k_stand      = 0.0)
     :name(k_name), nr(k_nr), stand(k_stand)    
     { }

     // Die get Methoden:
     const string& getName()  const { return name; }
     unsigned long getNr()    const { return nr; }
     double        getStand() const { return stand; }

     // Die set Methoden:
     bool setName( const string& s)
     {
        if( s.size() < 1)     // Kein leerer Name
          return false;
        name = s;
        return true; 
     }
     void setNr( unsigned long n) { nr = n; }
     void setStand(double x) { stand = x; }

     // Weitere Methoden:
     void display() const;

     istream& read( istream& is);
     ostream& write( ostream& os) const;
};

// display() zeigt die Konto-Daten am Bildschirm an.
inline void Konto::display() const 
{
   cout << fixed << setprecision(2)
        << "--------------------------------------\n"
        << "Kontoinhaber:  " << name  << '\n'
        << "Kontonummer:   " << nr    << '\n'
        << "Kontostand:    " << stand << '\n'
        << "--------------------------------------" << endl;
}

// write() schreibt die Daten des Kontos in den Stream os.
// Return-Wert: true, falls erfolgreich, sonst false.
inline ostream& Konto::write(ostream& os) const
{
   os << name << '\0';           // String schreiben
   os.write((char*)&nr, sizeof(nr) );
   os.write((char*)&stand, sizeof(stand) );

   return os;
}
 
// read ist die Gegenfunktion von write().
// read() liest die Daten eines Kontos aus dem Stream is und
// berschreibt damit die Daten im aktuellen Objekt.

inline istream& Konto::read(istream& is)
{
   getline( is, name, '\0');     // String einlesen    
   is.read( (char*)&nr, sizeof(nr) );
   is.read( (char*)&stand, sizeof(stand)); 

   return is;
}

#endif   //  _KONTO_
