// -------------------------------------------------------
// hash_dat.cpp : Die Methoden von HashEntry und HashFile
// -------------------------------------------------------

#include "hash_dat.h"

fstream& HashEntry::write(fstream& f) const
{
    f.write((char*)&nr, sizeof(nr) );
    f.write( name, sizeof(name) );

    return f;
}

fstream& HashEntry::read(fstream& f)
{
    f.read((char*)&nr, sizeof(nr) );
    f.read( name, sizeof(name)); 

    return f;
}

fstream& HashEntry::write_at(fstream& f, unsigned long pos) const
{
    f.seekp(pos);
    f.write((char*)&nr, sizeof(nr) );
    f.write( name, sizeof(name) );

    return f;
}

fstream& HashEntry::read_at(fstream& f, unsigned long pos)
{
    f.seekg(pos);
    f.read((char*)&nr, sizeof(nr) );
    f.read( name, sizeof(name)); 

    return f;
}

// ---------------------------------------------------
// Methoden der Klasse HashFile

HashFile::HashFile(const string file, unsigned long n) throw(open_error)
{ 
   ios::openmode mode =  ios::in | ios::out | ios::binary;

   f.open(file.c_str(), mode);  // ffnen, falls Datei schon existiert.

   if(!f)                       // Falls Datei nicht vorhanden
   {   
      f.clear();
      mode |= ios::trunc;
      f.open(file.c_str(), mode);
      if(!f)
         throw open_error(name);
   }
   name = file;
   b = n;

   HashEntry rec(0L, "");
   f.seekp(0L);
   for( unsigned long i=0; i < b; i++)    // Adressraum vorbelegen
   {
      rec.write(f);
      if(!f)
        throw write_error(name);
   }
}

void HashFile::insert( HashEntry& rec) throw( read_error, write_error)
{
    HashEntry temp;       
    int  size = temp.getSize();

    unsigned long pos = hash_func(rec.getNr());  // Hash-Wert.

    temp.read_at(f, pos*size);               // Slot einlesen.
    if(!f)
       throw read_error(name);
    else
    {
       if(temp.getNr() == 0L)          // Slot frei?   
           rec.write_at(f, pos*size);  // Ja   => In Datei schreiben. 
       else                            // Nein => Freien Slot suchen.
       {
          bool found = false; 

          unsigned long p = (pos*size + size)%(b*size);

          while( !found && p!= pos*size  )
          {   
             temp.read_at(f, p); 
             if(!f)
                 throw read_error(name);
             else 
                 if(temp.getNr() == 0L)      // Freier Slot gefunden.
                     found = true;
                 else
                     p = (p + size)%(b*size); // Zum nchsten Slot.
          }

          if( p == pos*size )                 // Adressraum voll.
              throw write_error(name);
   
          if( found == true )                // In Datei schreiben.
              rec.write_at(f,p);
       }
       
       if(!f)
           throw write_error(name);
    }
}

HashEntry& HashFile::retrieve( unsigned long key ) throw(read_error)
{
   static HashEntry temp; 
   int size = temp.getSize();

   unsigned long pos = hash_func(key);    // Hash-Wert.

   temp.read_at(f, pos*size);             // Slot einlesen.

   if(!f) throw read_error(name);

   if(temp.getNr() == key)                // Gefunden?
      return temp;                        // Ja   => fertig
   else                                   // Nein => suchen
   {
      unsigned long p = (pos*size + size) % (b*size);

      while( p!= pos *size )
      {   
         temp.read_at(f, p); 
         if(!f)
             throw read_error(name);
         else 
             if(temp.getNr() == key)      // Satz gefunden.
                return temp;
             else
                p = (p + size)%(b*size);  // Zum nchsten Slot.
      }

      temp.setNr(0L); temp.setName("");   // Schlssel existiert nicht.
      return temp;
   }
}

void HashFile::display()  throw( read_error )
{
    HashEntry temp;
    f.seekg(0L);
 
    for(unsigned int i = 0; i < b; i++)
    {
        temp.read(f);
        if(!f)
            throw read_error(name);
        temp.display();
    }
    
    f.clear();
}
