// -------------------------------------------------------
// WebAccount.cpp
// Definition von Methoden der Klasse UserTable
// zur passwortgeschtzten Benutzerverwaltung. 
// -------------------------------------------------------

#include "WebAccount.h"

ITERATOR UserTable::find(const string& pw, const string& s)
{
   auto er = ht.equal_range(hasher(pw));   // In diesem Bereich.
                                           // suchen.
   ITERATOR it = er.first;
   for( ; it != er.second; ++it) 
       if( (it->second).getUser() == s)   
         break;                        // Benutzerkonto gefunden.
   return it;
}

bool UserTable::isUser(const string& pw, const string& s)
{
   ITERATOR it = find(pw, s);

   return (it != ht.end()) ? true : false;
}

bool UserTable::addUser(const string& pw, const string& s)
{
   if( !isUser(pw, s))  // Einfgen, falls noch nicht vorhanden.
   {
      WebAccount user(s);
      ht.insert(pair<size_t, WebAccount>(hasher(pw), user));
      return true;
   }
   else
      return false;
}

bool UserTable::removeUser(const string& pw, const string& s)
{
   ITERATOR it  = find(pw, s);

   if(it != ht.end())
   {
      ht.erase(it);
      return true;
   }
   else
      return false;
}

bool UserTable::login(const string& pw, const string& s)
{
   ITERATOR it = find(pw, s);

   if(it != ht.end())              // Falls Webaccount gefunden:
   {
      it->second.setLogin(true);   // Login-Flag setzen.
      return true;
   }
   else
      return false;
}

bool UserTable::logout(const string& pw, const string& s)
{
   ITERATOR it = find(pw, s);

   if(it != ht.end())             // Falls Webaccount gefunden       
   {
      it->second.setLogin(false);
      return true;
   }
   else
      return false;
}

void UserTable::displayActiveUsers()
{
   for(ITERATOR it = ht.begin(); it != ht.end(); it++)
      if( it->second.getLogin())
          cout << it->second.getUser() << endl;
}
